import jenkins
import url

from datetime import datetime, timedelta
import time

ONE_HOUR_IN_MS = 1 * 60 * 60 * 1000


def get_boundary_timestamps(past_hours):
    end_timestamp = int(time.time() * 1000)
    start_timestamp = end_timestamp - int(past_hours) * ONE_HOUR_IN_MS
    return (start_timestamp, end_timestamp)


def build_in_boundary(past_hours, timestamp):
    start_timestamp, end_timestamp = get_boundary_timestamps(past_hours)
    return True if timestamp >= start_timestamp and timestamp < end_timestamp else False


def get_jenkins_home(job_url):
    myurl = url.parse(job_url)
    if myurl.port:
        return myurl.scheme + "://" + myurl.host + ":" + str(myurl.port)
    return myurl.scheme + "://" + myurl.host


def get_job_name(job_url):
    jenkins_home = get_jenkins_home(job_url)
    items = job_url.replace(jenkins_home, "").strip("/").split("job")
    return "/".join([i.strip("/") for i in items if i])


def get_jenkins_instance(job_url):
    jenkins_home = get_jenkins_home(job_url)
    return jenkins.Jenkins(jenkins_home)


def get_build_numbers(jenkins_instance, job_name):
    result = jenkins_instance.get_job_info(job_name)
    return [i["number"] for i in result["builds"]]


def get_test_reports(job_url, past_hours):
    test_reports = {}
    jenkins_instance = get_jenkins_instance(job_url)
    job_name = get_job_name(job_url)
    build_numbers = get_build_numbers(jenkins_instance, job_name)
    for number in build_numbers:
        build_info = jenkins_instance.get_build_info(job_name, number)
        if build_info["building"]:
            continue
        elif build_in_boundary(past_hours, build_info["timestamp"] + build_info["duration"]):
            report = jenkins_instance.get_build_test_report(job_name, number)
            test_reports[number] = {}
            test_reports[number]["duration"] = build_info["duration"]
            test_reports[number]["report"] = report
        else:
            break
    return test_reports



import numpy as np
import io

import matplotlib.pyplot as plt


def get_binary_figure(build_summary):
    N = len(build_summary)
    passed = [i["pass"] for i in build_summary]
    failed = [i["fail"] for i in build_summary]

    ind = np.arange(N)
    plt.figure(figsize=(8, 4.5))

    p1 = plt.bar(ind, passed, color="green")
    p2 = plt.bar(ind, failed, bottom=passed, color="red")

    plt.ylabel('Number of cases')
    plt.title('Build Trend')
    plt.xticks(ind, ["#%s" % build_summary[i]["number"]
                     for i in range(N)],  rotation='vertical')

    plt.legend((p1[0], p2[0]), ('Pass', 'Fail'))

    buf = io.BytesIO()
    plt.savefig(buf, format='png')
    buf.seek(0)
    return buf.read()



from .argument import get_parser
from .build import get_test_reports
from .summary import get_build_summary
from .summary import get_case_summary
from .report import get_html_output
from .mail import SendEmail
from . import __version__


def conport(args=None):
    parser = get_parser()
    args = parser.parse_args(args)

    if args.version:
        print("conport v" + __version__)
        return

    test_reports = get_test_reports(args.job_url, args.past_hours)
    if not test_reports:
        print("no valie test reports, exit")
        return
    build_summary = get_build_summary(test_reports)
    case_summary = get_case_summary(test_reports)

    html_output = get_html_output(
        args.job_url, args.report_title, args.past_hours, build_summary, case_summary, args.pure_html)
    if args.send_email == "true":
        SendEmail(args.mail_host, args.mail_user, args.mail_pwd, args.report_title, args.sender,
                  args.receivers, args.receivers_cc, html_output, build_summary, args.pure_html)


if __name__ == "__main__":
    conport()



import configparser
import os
from attrdict import AttrDict


def parse_config_file(config_file):
    config = configparser.ConfigParser()
    config.read(config_file)
    return AttrDict(dict(config._sections))


def get_default_config():
    config_file = os.path.join(os.path.abspath(
        os.path.dirname(__file__)), "cfg.ini")
    return parse_config_file(config_file)



from datetime import date
import smtplib

from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.MIMEImage import MIMEImage

from .summary import get_build_summary
from .figure import get_binary_figure


class SendEmail(object):
    def __init__(self, host, user, password, title, _from, to, cc, html_output, build_summary, pure_html):
        self.mail_host = host
        self.mail_user = user
        self.mail_password = password
        self.msg = None
        self._from = _from
        self.to = ';'.join([i.strip() for i in to.strip().split(",")])
        self.cc = ';'.join([i.strip() for i in cc.strip().split(",")])
        self.title = '[%s] %s' % (date.today(), title)
        self.pure_html = pure_html
        self.html_output = html_output
        self.build_summary = build_summary
        self.run()

    def run(self):
        self._generate_mail_text()
        self._send_email()

    def update_msg(self):
        self.msg["From"] = self._from
        self.msg["To"] = self.to
        self.msg["Cc"] = self.cc
        self.msg["Subject"] = self.title

    def _generate_mail_text(self):
        if self.pure_html == "true":
            msg = MIMEText(self.html_output, 'html', 'utf-8')
        else:
            msg = MIMEMultipart('related')
            msg.attach(MIMEText(self.html_output, 'html'))
            msgImage = MIMEImage(get_binary_figure(self.build_summary), "png")
            msgImage.add_header('Content-ID', '<build_summary_image>')
            msg.attach(msgImage)
        self.msg = msg
        self.update_msg()

    def _send_email(self):
        try:
            smtpobj = smtplib.SMTP()
            smtpobj.connect(self.mail_host, 25)
            try:
                smtpobj.login(self.mail_user, self.mail_password)
            except Exception:
                print("login fail, ignore.....")
                pass
            print(self.msg['From'], self.msg['To'], self.msg['Cc'])
            smtpobj.sendmail(self.msg['From'], (self.msg['To'] + ';' + self.msg['Cc']).split(';'),
                             self.msg.as_string())
            smtpobj.quit()
            print("sent email successfully!")
        except smtplib.SMTPException as e:
            raise RuntimeError(
                "can not send email. error is %s." % str(e))



import argparse

from .config import get_default_config


def get_parser():
    defaults = get_default_config()

    parser = argparse.ArgumentParser()

    parser.add_argument("--job_url", help="Jenkins job url",
                        default=defaults.default.job_url)
    parser.add_argument("--report_title", help="continuous testing report title",
                        default=defaults.default.report_title)
    parser.add_argument("--past_hours", help="number of past hours to be monitored",
                        type=int, default=defaults.default.past_hours)

    parser.add_argument("--send_email", help="whether to send email or not",
                        default=defaults.email.send_email)
    parser.add_argument("--mail_host", help="email host",
                        default=defaults.email.mail_host)
    parser.add_argument("--mail_user", help="email user",
                        default=defaults.email.mail_user)
    parser.add_argument("--mail_pwd", help="email password",
                        default=defaults.email.mail_pwd)
    parser.add_argument("--sender", help="email sender",
                        default=defaults.email.sender)
    parser.add_argument(
        "--receivers", help="email receivers, format is receiver1, receiver2, ...", default=defaults.email.receivers)
    parser.add_argument("--receivers_cc",
                        help="email receivers cc, format is receivercc1, receivercc2, ...", default=defaults.email.receivers_cc)

    parser.add_argument("--pure_html", help="pure html or not",
                        default=defaults.default.pure_html)

    parser.add_argument("--version", action='store_true', help="print version")

    return parser



test_reports = {
    100: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    101: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    102: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    103: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    104: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "FAILED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    105: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "FAILED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "FAILED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    106: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    107: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "FAILED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "FAILED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "FAILED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 39000
                        }
                    ]
                }
            ]
        }
    },
    108: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "FAILED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    109: {
        'duration':
            200000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "FAILED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 30000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 70000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 60000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    },
    110: {
        'duration':
            210000,
        'report': {
            'suites': [
                {
                    "name": "Login Wepage",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Loging with Correct Username and Password",
                            "duration": 20000
                        },
                        {
                            "status": "PASSED",
                            "name": "Loging with Incorrect Username and Password",
                            "duration": 60000
                        }
                    ]
                },
                {
                    "name": "Browser Commodity",
                    "cases": [
                        {
                            "status": "PASSED",
                            "name": "Browser Public Commodity",
                            "duration": 57000
                        },
                        {
                            "status": "PASSED",
                            "name": "Browser Private Commodity",
                            "duration": 40000
                        }
                    ]
                }
            ]
        }
    }
}



from __future__ import division
import datetime
import os

from jinja2 import Template


def get_template_path():
    return os.path.join(os.path.abspath(os.path.dirname(__file__)), "template.html")


def get_html_output(job_url, report_title, past_hours, build_summary, case_summary, pure_html):
    jinja2_template_string = open(get_template_path(), 'rb').read()
    template = Template(jinja2_template_string)

    build_metrics = {}
    build_metrics["avg_duration"] = round(sum(
        [i["duration"] for i in build_summary])/len(build_summary), 1) if len(build_summary) else "NA"
    passed_builds = [i["duration"] for i in build_summary if i["fail"] == 0]
    build_metrics["avg_duration_pass"] = "NA" if not passed_builds else round(sum(passed_builds
                                                                                  )/len(passed_builds), 1)
    build_metrics["max_duration"] = round(
        max([i["duration"] for i in build_summary]), 1) if [i["duration"] for i in build_summary] else "NA"
    build_metrics["max_pass_duration"] = round(max(
        passed_builds), 1) if passed_builds else "NA"

    html_template_string = template.render(build_metrics=build_metrics,
                                           job_url=job_url, report_title=report_title, past_hours=past_hours, case_summary=case_summary, now=datetime.datetime.utcnow(), build_summary=build_summary, pure_html=True if pure_html == "true" else False)

    return html_template_string




from __future__ import division
import collections


def get_atom_reports(test_reports):
    atom_reports = []
    for number, report in test_reports.iteritems():
        suites = report['report']['suites']
        for suite in suites:
            suite_name = suite['name']
            suite_cases = suite['cases']
            for suite_case in suite_cases:
                atom_reports.append(
                    (number, suite_name, suite_case['name'], suite_case['status'], suite_case['duration']))
    return atom_reports


def get_merged_reports(atom_reports):
    merged_reports = {}
    for atom in atom_reports:
        full_name = '.'.join([atom[1], atom[2]])
        if full_name not in merged_reports:
            merged_reports[full_name] = {
                'atoms': [(atom[0], atom[3], atom[4])]}
        else:
            merged_reports[full_name]['atoms'].append(
                (atom[0], atom[3], atom[4]))
    return merged_reports


def get_ordered_reports(merged_reports):
    for full_name, merged in merged_reports.iteritems():
        merged['suite'] = full_name.split('.')[0]
        merged['case'] = full_name.split('.')[1]
        merged['pass'] = len([i for i in merged['atoms'] if i[1] == 'PASSED'])
        merged['total'] = len(merged['atoms'])
        merged['npass'] = merged['total'] - merged['pass']
        merged['fail_rate'] = merged['npass'] / merged['total']
        merged['npass_numbers'] = [i[0]
                                   for i in merged['atoms'] if i[1] != 'PASSED']
        sum_pass_duration = sum([i[2]
                                 for i in merged['atoms'] if i[1] == 'PASSED'])
        merged['avg_pass_duration'] = round(
            sum_pass_duration / 1000 / merged['pass'], 1) if merged['pass'] else "NA"
        sum_duration = sum([i[2] for i in merged['atoms']])
        merged['avg_duration'] = round(sum_duration/1000 / merged['total'], 1)
        merged['max_duration'] = round(
            max([i[2]/1000 for i in merged['atoms']]), 1)
        merged['max_pass_duration'] = round(max([i[2]/1000
                                                 for i in merged['atoms'] if i[1] == 'PASSED']), 1) if merged['pass'] else "NA"

    sorted_merged_reports = sorted(
        merged_reports.items(), key=lambda x: (-1*x[1]['fail_rate'], x[1]["suite"], x[1]["case"]))

    return collections.OrderedDict(sorted_merged_reports)


def get_build_summary(test_reports):
    build_summary = []
    for number, report in test_reports.iteritems():
        build = {"number": number, "pass": 0,
                 "fail": 0, "duration": round(report["duration"]/1000, 1)}
        suites = report['report']['suites']
        for suite in suites:
            for case in suite['cases']:
                if case["status"] == "PASSED":
                    build["pass"] += 1
                else:
                    build["fail"] += 1
        build_summary.append(build)
    sorted(build_summary, key=lambda x: x["number"])
    return build_summary


def get_case_summary(test_reports):
    case_summary = get_ordered_reports(
        get_merged_reports(get_atom_reports(test_reports))).values()
    for i in range(len(case_summary)):
        case_summary[i]["rank"] = i + 1
    return case_summary



"""conport - Python generator of continuous testing report
"""
import io
import sys

import setuptools
from distutils.core import setup

def get_refined_long_descrition():
    description = io.open('README.md', encoding='utf8').read()[0]
    return (description[description.find("# conport"):], )


version = '1.1.3'

setup(
    include_package_data=True,
    name='conport',
    version=version,
    packages=['conport'],
    install_requires=[
        "jinja2>=2.7",
        "matplotlib",
        "python-jenkins",
        "url"
    ],
    long_description=get_refined_long_descrition(),
    entry_points={
        'console_scripts': [
            'conport = conport.conport:conport',
        ]
    },
    url='https://github.com/slxiao/contport',
    license='MIT',
    author='slxiao',
    author_email='shliangxiao@gmail.com',
    description='Generate continuous testing report',
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    data_files=[('', ['conport/cfg.ini', 'conport/template.html'])]
)
